import axios, { AxiosInstance } from 'axios';

/**
 *
 * @param corPID 公司的id
 * @param corSecret 你的应用的id
 * @param agentid 你的应用的id
 */
export class WechatWorkBase {
  corPID: string;
  corSecret: string;
  accessToken: string = '';
  accessTokenExpireAt: number = 0;
  agentid: number = 0;
  accessTokenURL: string = 'https://qyapi.weixin.qq.com/cgi-bin/gettoken';
  messageSendURL: string = 'https://qyapi.weixin.qq.com/cgi-bin/message/send';
  private isUpdating: Boolean = false;
  axios: AxiosInstance;
  constructor(corPID: string, corSecret: string, agentid: number) {
    this.corPID = corPID;
    this.corSecret = corSecret;
    this.agentid = agentid;
    this.axios = axios;
  }
  /**
   * 获取token并且会存储缓存
   */
  async GetAccessToken(): Promise<string> {
    const currentDate = +new Date();
    // 当前时间大于过期时间且未处于正在刷新状态
    if (this.accessTokenExpireAt < currentDate || this.isUpdating === false) {
      // 正在处于刷新 防止单个实例并发
      this.isUpdating = true;
      const params = { corpid: this.corPID, corpsecret: this.corSecret };
      try {
        const {
          data: { errcode, access_token, expires_in }
        } = await this.axios.get(this.accessTokenURL, { params });
        if (errcode === 0) {
          this.accessToken = access_token;
          this.accessTokenExpireAt = currentDate + expires_in * 1000 - 20000;
        }
      } catch (error) {
        console.log(error);
      }
    }
    // 解除更新状态
    this.isUpdating = false;
    return this.accessToken;
  }
}
