import { WechatWorkBase } from './base';
import {
  MessageBody,
  SendResponse,
  MessageBodyText,
  MessageBodyMarkdown
} from '../types/message';

export default class WechatWorkPush extends WechatWorkBase {
  /**
   * 最简单版本的发送消息 什么都不管你 自己玩
   *
   * @param data {MessageBody} 发送主体
   */
  public async PushMessage(data: MessageBody): Promise<SendResponse> {
    // 我要给你写业务逻辑吗...
    const token = await this.GetAccessToken();

    const { data: res } = await this.axios.post(
      `${this.messageSendURL}?access_token=${token}`,
      data
    );
    return res;
  }
  /**
   * 发送文本消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} param0.text.content
   * 消息内容，最长不超过2048个字节，超过将截断
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageText(data: MessageBodyText): Promise<SendResponse> {
    const { touser = '@all', toparty = '', totag = '', safe = 0, text } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'text',
      text
    });
    return res;
  }

  /**
   * 发送Markdown文本消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} param0.text.content
   * 消息内容，最长不超过2048个字节，超过将截断
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageMarkdown(
    data: MessageBodyMarkdown
  ): Promise<SendResponse> {
    const {
      touser = '@all',
      toparty = '',
      totag = '',
      safe = 0,
      markdown
    } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'markdown',
      markdown
    });
    return res;
  }
  /**
   * 发送图片消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} param0.image.media_id
   * 媒体文件id，可以调用上传临时素材接口获取
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageImage(data: MessageBody): Promise<SendResponse> {
    const { touser = '@all', toparty = '', totag = '', safe = 0, image } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'image',
      image
    });
    return res;
  }
  /**
   * 发送语音消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} param0.voice.media_id
   * 媒体文件id，可以调用上传临时素材接口获取
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageVoice(data: MessageBody): Promise<SendResponse> {
    const { touser = '@all', toparty = '', totag = '', safe = 0, image } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'voice',
      image
    });
    return res;
  }

  /**
   * 发送视频消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} param0.video.media_id
   * 媒体文件id，可以调用上传临时素材接口获取
   *
   * @param {string} [param0.video.title]
   * 消息标题
   *
   * @param {string} [param0.video.description]
   * 消息描述
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageVideo(data: MessageBody): Promise<SendResponse> {
    const { touser = '@all', toparty = '', totag = '', safe = 0, video } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'video',
      video
    });
    return res;
  }
  /**
   * 发送视频消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} param0.file.media_id
   * 媒体文件id，可以调用上传临时素材接口获取
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageFile(data: MessageBody): Promise<SendResponse> {
    const { touser = '@all', toparty = '', totag = '', safe = 0, file } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'file',
      file
    });
    return res;
  }
  /**
   * 发送文本卡片消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} param0.textcard.title
   * 消息标题
   *
   * @param {string} param0.textcard.description
   * 消息描述
   *
   * @param {string} param0.textcard.url
   * 消息跳转地址
   *
   * @param {string} [param0.textcard.btntxt="详情"]
   * 消息跳转地址文本描述
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageTextcard(data: MessageBody): Promise<SendResponse> {
    const {
      touser = '@all',
      toparty = '',
      totag = '',
      safe = 0,
      textcard
    } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'textcard',
      textcard
    });
    return res;
  }

  /**
   * 发送图文消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {Object} param0.news
   * 图文消息new属性
   *
   * @param {Array} param0.news.articles
   * 图文列表
   *
   * @param {string} param0.news.articles.title
   * 图文列表元素标题
   *
   * @param {string} [param0.news.articles.description]
   * 图文列表元素描述
   *
   * @param {string} param0.news.articles.url
   * 图文列表元素点击后跳转的链接。
   *
   * @param {string} [param0.news.articles.picurl]
   * 图文列表元素图片链接。
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageNews(data: MessageBody): Promise<SendResponse> {
    const { touser = '@all', toparty = '', totag = '', safe = 0, news } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'news',
      news
    });
    return res;
  }

  /**
   * 发送资源存储于企业微信的图文消息
   *
   * @param {object} param0 touser toparty totag 不传值默认发送给所有人
   *
   * @param {string} [param0.touser='@all'] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {Object} param0.mpnews
   * 图文消息new属性
   *
   * @param {Array} param0.mpnews.articles
   * 图文列表
   *
   * @param {string} param0.mpnews.articles.title
   * 图文列表元素标题
   *
   * @param {string} param0.mpnews.articles.thumb_media_id
   * 图文列表元素缩略图id
   *
   * @param {string} param0.mpnews.articles.content
   * 图文列表元素内容
   *
   * @param {string} [param0.mpnews.articles.digest]
   * 图文列表元素描述
   *
   * @param {string} [param0.mpnews.articles.author]
   * 图文列表元素作者
   *
   * @param {string} [param0.mpnews.articles.content_source_url]
   * 图文列表元素点击后跳转的链接。
   *
   * @param {number} param0.safe
   * 表示是否是保密消息，0表示否，1表示是，默认0
   */
  public async PushMessageMPnews(data: MessageBody): Promise<SendResponse> {
    const {
      touser = '@all',
      toparty = '',
      totag = '',
      safe = 0,
      mpnews
    } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      safe,
      agentid: this.agentid,
      msgtype: 'mpnews',
      mpnews
    });
    return res;
  }
  /**
   * 发送小程序推送消息 / 只能小程序类型的推送 不支持全员推送
   *
   * @param {object} param0 touser toparty totag
   *
   * @param {string} [param0.touser] - 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为 "@all"，则向关注该企业应用的全部成员发送
   *
   * @param {string} [param0.toparty]
   * 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {string} [param0.totag]
   * 标签ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为 "@all" 时忽略本参数
   *
   * @param {Object} param0.miniprogram_notice
   * 小程序消息
   *
   * @param {string} param0.miniprogram_notice.title
   * 消息标题
   *
   * @param {string} param0.miniprogram_notice.appid
   * 小程序id
   *
   * @param {string} [param0.miniprogram_notice.page]
   * 点击消息卡片后的小程序页面，仅限本小程序内的页面。该字段不填则消息点击后不跳转。
   *
   * @param {string} [param0.miniprogram_notice.description]
   * 消息描述
   *
   * @param {Boolean} [param0.miniprogram_notice.emphasis_first_item]
   * 是否放大第一个
   *
   * @param {Array} [param0.miniprogram_notice.content_item]
   * 消息键值对列表 [{"key":"value"}]
   *
   */
  public async PushMessageMiniProgram(
    data: MessageBody
  ): Promise<SendResponse> {
    const {
      touser = '@all',
      toparty = '',
      totag = '',
      miniprogram_notice
    } = data;
    const res = await this.PushMessage({
      touser,
      toparty,
      totag,
      msgtype: 'miniprogram_notice',
      miniprogram_notice
    });
    return res;
  }
}
