/**
 * MessageBody 服务端消息推送的主体内容
 * 需要重构
 */

export interface MessageBody {
  touser?: string;
  toparty?: string;
  totag?: string;
  msgtype?: string;
  agentid?: number;
  text?: Text;
  image?: Media;
  voice?: Media;
  video?: Video;
  file?: File;
  textcard?: Textcard;
  news?: News;
  mpnews?: Mpnews;
  markdown?: Text;
  miniprogram_notice?: MiniprogramNotice;
  safe?: number;
}

export interface MessageBase {
  touser?: string;
  toparty?: string;
  totag?: string;
  safe?: number;
}

export interface MessageBodyText extends MessageBase {
  text: Text;
}

export interface MessageBodyMarkdown extends MessageBase {
  markdown: Text;
}

/**
 * Text 是推送文本内容的详情内容
 * @param content 文本内容
 */

export interface Text {
  content: string;
}
/**
 * Media 是推送语音消息的主体
 *
 * @param  media_id 语音文件id
 */
export interface Media {
  media_id: string;
}
/**
 * Video 是推送视频消息的主体
 *
 * @param  media_id 视频文件id
 * @param  [title] 消息标题
 * @param  [description] 消息描述
 */
export interface Video {
  media_id: string;
  title?: string;
  description?: string;
}

/**
 * File 是推送文件消息的主体
 *
 * @param  media_id 文件id
 */
export interface File {
  media_id: string;
}

/**
 * Textcard 是推送文本卡片的主体
 *
 * @param  title 文本卡片标题
 * @param  description 文本描述
 * @param  url 跳转地址
 * @param  [btntxt] 按钮文本
 */
export interface Textcard {
  title: string;
  description: string;
  url: string;
  btntxt?: string;
}

/**
 * News 是图文消息的主体
 */
export interface News {
  articles: Articles[];
}

export interface Articles {
  title: string;
  description?: string;
  url: string;
  picurl?: string;
}

/**
 * Mpnews 是图文消息的主体消息存储在微信
 */
export interface Mpnews {
  articles: MArticles[];
}

export interface MArticles {
  title: string;
  thumb_media_id: string;
  author?: string;
  content_source_url?: string;
  content?: string;
  digest?: string;
  safe?: Number;
}

/**
 * MiniprogramNotice 小程序消息
 */

export interface MiniprogramNotice {
  appid: String;
  page?: String;
  title: String;
  description?: String;
  emphasis_first_item?: Boolean;
  content_item?: { key: string; value: string }[];
}

export interface SendResponse {
  errcode: Number;
  errmsg: String;
  invaliduser?: String;
}
